﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using ZedGraph;
using System.Speech.Recognition;
using System.Threading;
using System.Speech.Synthesis;



namespace UI
{   
    // Initialization
    public partial class Form1 : Form
    {
        int tickStart = 0;
        int intMode = 1;  // Default mode in Farenheit display

        const int maxTempC = 85;     // Max cooler operation range in Celcius 
        const int minTempC = -10;

        const int maxTempF = 185;   // Max cooler operation range in Farenheit
        const int minTempF = 14;


        public Form1()
        {
            InitializeComponent();
        }


        // Application form initilazation
        private void Form1_Load(object sender, EventArgs e)
        {

            trackBar1.Maximum = maxTempF;  // track bar initialization
            trackBar1.Minimum = minTempF;

            // Zed graph initialization, used for temperature data plot
            GraphPane myPaneF = zedGraphControl1.GraphPane;
            myPaneF.Title.Text = "Temperature Controller\n" + "(After 25 seconds the graph scrolls)";
            myPaneF.XAxis.Title.Text = "Time, Seconds";
            myPaneF.YAxis.Title.Text = "Temperature";

            GraphPane myPaneC = zedGraphControl2.GraphPane;
            myPaneC.Title.Text = "Temperature Controller\n" + "(After 25 seconds the graph scrolls)";
            myPaneC.XAxis.Title.Text = "Time, Seconds";
            myPaneC.YAxis.Title.Text = "Temperature";

            RollingPointPairList list_1F = new RollingPointPairList(1200);
            RollingPointPairList list_2F = new RollingPointPairList(1200);
            RollingPointPairList list_1C = new RollingPointPairList(1200);
            RollingPointPairList list_2C = new RollingPointPairList(1200);

            LineItem curve_1F = myPaneF.AddCurve("Desired Temperature", list_1F, Color.Red, SymbolType.None);
            LineItem curve_2F = myPaneF.AddCurve("Current Temperature", list_2F, Color.Blue, SymbolType.None);
            LineItem curve_1C = myPaneC.AddCurve("Desired Temperature", list_1C, Color.Red, SymbolType.None);
            LineItem curve_2C = myPaneC.AddCurve("Current Temperature", list_2C, Color.Blue, SymbolType.None);

            zedGraphControl1.Visible = true;
            zedGraphControl2.Visible = false;


            myPaneF.XAxis.Scale.Min = 0;
            myPaneF.XAxis.Scale.Max = 30;
            myPaneF.XAxis.Scale.MinorStep = 1;
            myPaneF.XAxis.Scale.MajorStep = 5;

            myPaneC.XAxis.Scale.Min = 0;
            myPaneC.XAxis.Scale.Max = 30;
            myPaneC.XAxis.Scale.MinorStep = 1;
            myPaneC.XAxis.Scale.MajorStep = 5;


            zedGraphControl1.AxisChange();
            zedGraphControl2.AxisChange();

            // Button initialization
            button2.Enabled = false;
            button3.Enabled = false;
            textBox2.Enabled = false;
            textBox3.Enabled = false;

            // Button and label bring to the front
            label8.BringToFront();
            button6.BringToFront();
        }

        // Override resize function
        private void Form1_Resize(object sender, EventArgs e)
        {
            SetSize();
        }

        // Set the size and location of the ZedGraphControl
        private void SetSize()
        {
            // Control is always 10 pixels inset from the client rectangle of the form
            Rectangle formRect = this.ClientRectangle;
            formRect.Inflate(-10, -10);

            if (zedGraphControl1.Size != formRect.Size)
            {
                zedGraphControl1.Location = formRect.Location;
                zedGraphControl1.Size = formRect.Size;
            }
        }

        //Connection setting button
        private void button1_Click(object sender, EventArgs e)
        {
            string[] ports = SerialPort.GetPortNames();
            comboBox1.Items.Clear();
            foreach (string port in ports)
                comboBox1.Items.Add(port);
        }

        string t;
        SerialPort sp;

        // Read button
        private void button2_Click(object sender, EventArgs e)
        {
            button3.Enabled = true;
            textBox2.Enabled = true;
            textBox3.Enabled = true;
            tickStart = Environment.TickCount;      // Timer starts when reading starts 
            t = comboBox1.Text.ToString();
            sErial(t);
            button2.Enabled = false;
        }

        // Serial communication Initialization with Mbed used for data receiving
        void sErial(string Port_name)
        {
            sp = new SerialPort(Port_name, 9600, Parity.None, 8, StopBits.One);
            sp.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler);
            sp.Open();
            sp.DiscardInBuffer();
        }

        string w, sendText = "70";

        // Parse received data which contains four different mode object tempertaure, PWM, fan speed and heat sink temperature
        const int TEMP = 1;
        const int PWM = 2;
        const int FAN = 3;
        const int HEAT = 4;
        int receiveMode = TEMP;
        private string ParseReceivedString(string x, string parser)
        {
            int first = x.IndexOf(parser) + parser.Length;
            if (first == (-1 + parser.Length))
            {             
                    return x;
            }           
            
            if (parser == "CurTemp ")
                receiveMode = TEMP;
            if (parser == "PwmOut ")
                receiveMode = PWM;
            if (parser == "FanS ")
                receiveMode = FAN;
            if (parser == "HeatTemp ")
                receiveMode = HEAT;

            string y = x.Substring(first, x.Length - first) + '\n';
            return y;
        }


 //      data received function handerler 
        private void DataReceivedHandler(object sender, SerialDataReceivedEventArgs e)
        {

            SerialPort spp = (SerialPort)sender;
            w = "";
            w = spp.ReadLine();

            w = ParseReceivedString(w, "CurTemp ");
            w = ParseReceivedString(w, "PwmOut ");
            w = ParseReceivedString(w, "FanS ");
            w = ParseReceivedString(w, "HeatTemp ");

//            Invoke(new Action(() => textBox6.AppendText(w+"\n")));
 
            if (w != String.Empty)
            {
                if (receiveMode == TEMP)
                {
                    double temperatureInC = Convert.ToDouble(w);
                    double temperatureInF = temperatureInC * 9 / 5 + 32;
                    w = string.Format("{0:0.0}", temperatureInF);

                    Draw(sendText, w);

                    if (tempMode == 1)
                    {
                        double temp = Convert.ToDouble(w);
                        temp = (temp - 32) * 5 / 9;
                        w = string.Format("{0:0.0}", temp);
                    }

                    Invoke(new Action(() => textBox2.Clear()));
                    Invoke(new Action(() => textBox2.AppendText(w)));


                }
                else if (receiveMode == PWM)
                {
                    int temp = Convert.ToInt16(w);
                    int absTemp = Math.Abs(temp);
                    groupBox6.Invoke((MethodInvoker)(() => groupBox6.Text = absTemp.ToString() + "%"));
                    label8.Invoke((MethodInvoker)(() => label8.Width = Convert.ToInt32((double)absTemp / 100 * (groupBox6.Width - 10))));

                    double currentT = Convert.ToDouble(w);
                    double desiredT = Convert.ToDouble(sendText);
                    if (temp > 0.1)
                    {
                        Invoke(new Action(() => textBox3.Clear()));
                        Invoke(new Action(() => textBox3.AppendText("Cooling\n")));
                    }
                    else if (temp < -0.1)
                    {
                        Invoke(new Action(() => textBox3.Clear()));
                        Invoke(new Action(() => textBox3.AppendText("Heating\n")));

                    }
                    else
                    {
                        Invoke(new Action(() => textBox3.Clear()));
                        Invoke(new Action(() => textBox3.AppendText("Idling\n")));                    
                    }


                }

                else if (receiveMode == FAN)
                {
                    Invoke(new Action(() => textBox6.Clear()));
                    Invoke(new Action(() => textBox6.AppendText(w)));
                }

                else if (receiveMode == HEAT)

                {
                    Invoke(new Action(() => textBox7.Clear()));
                    Invoke(new Action(() => textBox7.AppendText(w + "°C")));
                }

            }
        }

        //Draw 4 curve (desired temperature in F and in C, Current temperature in F and in C)
        private void Draw(string setpoint, string current)
        {
            double intsetpoint;
            double intcurrent;
            double setpointC;

            double.TryParse(setpoint, out intsetpoint);
            double.TryParse(current, out intcurrent);

            double currentC = (intcurrent - 32) * 5 / 9;
            string wcC = string.Format("{0:0.0}", currentC);
            double.TryParse(wcC, out currentC);

            if (tempMode == 0)
            {
                setpointC = (intsetpoint - 32) * 5 / 9;
                string wC = string.Format("{0:0.0}", setpointC);
                double.TryParse(wC, out setpointC);
            }
            else
            {
                setpointC = intsetpoint;

                intsetpoint = intsetpoint * 9 / 5 + 32;
                string wF = string.Format("{0:0.0}", intsetpoint);
                double.TryParse(wF, out intsetpoint);
            }



            // Make sure that the curvelist has at least one curve
            if (zedGraphControl1.GraphPane.CurveList.Count <= 0)
                return;

            if (zedGraphControl2.GraphPane.CurveList.Count <= 0)
                return;
            // Get the first CurveItem in the graph
            LineItem curve_1F = zedGraphControl1.GraphPane.CurveList[0] as LineItem;
            LineItem curve_2F = zedGraphControl1.GraphPane.CurveList[1] as LineItem;
            LineItem curve_1C = zedGraphControl2.GraphPane.CurveList[0] as LineItem;
            LineItem curve_2C = zedGraphControl2.GraphPane.CurveList[1] as LineItem;

            if (curve_1F == null)
                return;
            if (curve_2F == null)
                return;


            if (curve_1C == null)
                return;
            if (curve_2C == null)
                return;

            // Get the PointPairList
            IPointListEdit list_1F = curve_1F.Points as IPointListEdit;
            IPointListEdit list_2F = curve_2F.Points as IPointListEdit;
            IPointListEdit list_1C = curve_1C.Points as IPointListEdit;
            IPointListEdit list_2C = curve_2C.Points as IPointListEdit;
            // If this is null, it means the reference at curve.Points does not
            // support IPointListEdit, so we won't be able to modify it
            if (list_1F == null)
                return;
            if (list_2F == null)
                return;
            if (list_1C == null)
                return;
            if (list_2C == null)
                return;

            // Time is measured in seconds
            double time = (Environment.TickCount - tickStart) / 1000.0;

            list_1F.Add(time, intsetpoint);
            list_2F.Add(time, intcurrent);
            list_1C.Add(time, setpointC);
            list_2C.Add(time, currentC);

            // Keep the X scale at a rolling 30 second interval, with one
            // major step between the max X value and the end of the axis
            Scale xScale = zedGraphControl1.GraphPane.XAxis.Scale;
            if (time > xScale.Max - xScale.MajorStep)
            {
                if (intMode == 1)
                {
                    xScale.Max = time + xScale.MajorStep;
                    xScale.Min = xScale.Max - 30.0;
                }
                else
                {
                    xScale.Max = time + xScale.MajorStep;
                    xScale.Min = 0;
                }
            }

            Scale xScale2 = zedGraphControl2.GraphPane.XAxis.Scale;
            if (time > xScale2.Max - xScale2.MajorStep)
            {
                if (intMode == 1)
                {
                    xScale2.Max = time + xScale2.MajorStep;
                    xScale2.Min = xScale2.Max - 30.0;
                }
                else
                {
                    xScale2.Max = time + xScale2.MajorStep;
                    xScale2.Min = 0;
                }
            }


            // Make sure the Y axis is rescaled to accommodate actual data
            zedGraphControl1.AxisChange();
            zedGraphControl2.AxisChange();
            // Force a redraw
            zedGraphControl1.Invalidate();
            zedGraphControl2.Invalidate();
        }

        // Send Desired Tempertaure always in F (Only allow 3 digits number converting to string and a terminal character)
        private void button3_Click(object sender, EventArgs e)
        {
            int temp = Convert.ToInt32(textBox1.Text);
            if (tempMode == 0)
            {
                if (temp > maxTempF)
                    temp = maxTempF;
                else if (temp < minTempF)
                    temp = minTempF;
            }
            else
            {
                if (temp > maxTempC)
                    temp = maxTempC;
                else if (temp < minTempC)
                    temp = minTempC;
            }
            textBox1.Text = Convert.ToString(temp);
            trackBar1.Value = temp;

            sendText = textBox1.Text.ToString();
            while (sendText.Length < 3)
                sendText += ' ';
            textBox1.Text = sendText;
            if (tempMode == 0)                                             //tempMode = 1 stands for °C
            {
                double tempF = Convert.ToDouble(sendText);
                double tempC = (tempF - 32) * 5 / 9;
                string sendTextC = Convert.ToString(Math.Round(tempC));
                while (sendTextC.Length < 3)
                    sendTextC += ' ';
                sp.Write(sendTextC);
            }
            else
                sp.Write(sendText);

        }

        // Set Desired Temperature
        private void textBox1_TextChanged(object sender, EventArgs e)
        {   

        }

        // Enable read button after select corrected port 
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            button2.Enabled = true;
        }

        // add temperature by 1 button
        private void button4_Click(object sender, EventArgs e)
        {
            int temp = Convert.ToInt16(textBox1.Text);
            temp++;
            textBox1.Text = Convert.ToString(temp);
        }

        // decrease temperature by 1 button
        private void button5_Click(object sender, EventArgs e)
        {
            int temp = Convert.ToInt16(textBox1.Text);
            temp--;
            textBox1.Text = Convert.ToString(temp);
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        // Temperature mode = 0 means display in Farenheit
        int tempMode = 0;
        // global variable used for transform bewtween F and C
        double tempInC, tempInF;

        // Radio button change call back handler function. This button means display in farenheit
        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton1.Checked)
            {
                tempMode = 0;
                tempInC = Convert.ToDouble(textBox1.Text);
                tempInF = tempInC * 9 / 5 + 32;
                textBox1.Text = Convert.ToString(Math.Round(tempInF));
                label1.Text = "°F";
                label5.Text = "°F";
                label6.Text = "°F";

                trackBar1.Maximum = maxTempF;
                trackBar1.Minimum = minTempF;


                zedGraphControl1.Visible = true;
                zedGraphControl2.Visible = false;

                int temp = Convert.ToInt16(textBox1.Text);
                if (temp > maxTempF)
                    trackBar1.Value = maxTempF;

                else if (temp < minTempF)
                    trackBar1.Value = minTempF;
                else
                    trackBar1.Value = temp;

                sendText = textBox1.Text.ToString();
                while (sendText.Length < 3)
                    sendText += ' ';
                textBox1.Text = sendText;

            }
        }

        // Radio button change call back handler function. This button means display in Celcius
        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton2.Checked)
            {
                tempMode = 1;
                tempInF = Convert.ToDouble(textBox1.Text);
                tempInC = (tempInF - 32) * 5 / 9;
                textBox1.Text = Convert.ToString(Math.Round(tempInC));
                label1.Text = "°C";
                label5.Text = "°C";
                label6.Text = "°C";

                trackBar1.Maximum = maxTempC;
                trackBar1.Minimum = minTempC;


                zedGraphControl2.Visible = true;
                zedGraphControl1.Visible = false;

                int temp = Convert.ToInt16(textBox1.Text);
                if (temp > maxTempC)
                    trackBar1.Value = maxTempC;

                else if (temp < minTempC)
                    trackBar1.Value = minTempC;
                else
                    trackBar1.Value = temp;

                sendText = textBox1.Text.ToString();
                while (sendText.Length < 3)
                    sendText += ' ';
                textBox1.Text = sendText;

            }
        }

        // Pb mode click to change between Scroll graph and compact graph
        private void Pb_Mode_Click(object sender, EventArgs e)
        {
            if (Pb_Mode.Text == "SCROLL")
            {
                intMode = 1;
                Pb_Mode.Text = "COMPACT";
            }
            else
            {
                intMode = 0;
                Pb_Mode.Text = "SCROLL";
            }
        }

        // exit the program
        private void Pb_Exit_Click(object sender, EventArgs e)
        {
            Close();
        }


        private void zedGraphControl1_Load(object sender, EventArgs e)
        {

        }

        private void toolStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void groupBox2_Enter(object sender, EventArgs e)
        {

        }

        private void textBox2_TextChanged(object sender, EventArgs e)
        {

        }

        private void label5_Click(object sender, EventArgs e)
        {

        }

        private void label6_Click(object sender, EventArgs e)
        {

        }

        // text box that moniter the current cooler state(cooling, heating or iding)
        private void textBox3_TextChanged(object sender, EventArgs e)
        {
            if (textBox3.Text == "Cooling\n")
            {
                pictureBox1.Image = Image.FromFile("P:\\UI\\down.png");
                textBox3.ForeColor = System.Drawing.Color.Blue;
                label8.BackColor = System.Drawing.Color.Blue;
            }

            if (textBox3.Text == "Heating\n")
            {
                textBox3.ForeColor = System.Drawing.Color.Red;
                label8.BackColor = System.Drawing.Color.Red;
                pictureBox1.Image = Image.FromFile("P:\\UI\\up.png");
            }
            if (textBox3.Text == "Idling\n")
            {
                textBox3.ForeColor = System.Drawing.Color.Black;
                label8.BackColor = System.Drawing.Color.Black;
                pictureBox1.Image = Image.FromFile("P:\\UI\\check.png");
            }
        }


        private void pictureBox1_Click(object sender, EventArgs e)
        {

        }

        // Track bar change call back function
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            textBox1.Text = Convert.ToString(trackBar1.Value);
        }

        private void zedGraphControl2_Load(object sender, EventArgs e)
        {

        }

        private void richTextBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private void groupBox3_Enter(object sender, EventArgs e)
        {

        }

        private void groupBox5_Enter(object sender, EventArgs e)
        {

        }

        // Speech synthesis unit
        SpeechSynthesizer sSynth = new SpeechSynthesizer();
        SpeechRecognitionEngine sRecog = new SpeechRecognitionEngine();

        private void button6_Click(object sender, EventArgs e)
        {
            string state;
            if (label5.Text == "°F")
                state = " degrees Fahrenheit, and Now " + textBox3.Text + ".";
            else
                state = " degrees Celsius, and Now " + textBox3.Text + ".";
            sSynth.SpeakAsync("The current temperature is " + textBox2.Text + state);
        }

        //  Speech synthesis grammer setting
        private void button7_Click(object sender, EventArgs e)
        {
            System.Media.SystemSounds.Exclamation.Play();
            button7.Enabled = false;
            button8.Enabled = true;

            Choices sList = new Choices();
            sList.Add(new string[] { "zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "set", "clear", "exit", "stop"});
            Grammar gr = new Grammar(new GrammarBuilder(sList));
            try
            {
                sRecog.RequestRecognizerUpdate();
                sRecog.LoadGrammar(gr);
                sRecog.SpeechRecognized += sRecog_SpeechRecognized;
                sRecog.SetInputToDefaultAudioDevice();
                sRecog.RecognizeAsync(RecognizeMode.Multiple);
            }
            catch
            {
                return;
            }
        }

        // Speech recognition unit
        private void sRecog_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            if (e.Result.Text == "exit")
            {
                Application.Exit();
            }

            if (e.Result.Text == "stop")
            {
                button8.PerformClick();
                return;
            }

            if (e.Result.Text == "clear")
            {
                textBox4.Clear();
                return;
            }
            if (e.Result.Text == "set")
            {
                if (textBox4.Text != "")
                {
                    textBox1.Text = textBox4.Text;
                }
                textBox4.Text = "";
                button3.PerformClick();
                return;
            }
            string[] numList = new string[] { "zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine" };
            for (int i = 0; i < 10; i++)
            {
                if (e.Result.Text == numList[i] && textBox4.Text.Length < 3)
                {
                    textBox4.Text = textBox4.Text + Convert.ToString(i);
                    textBox4.Select(textBox4.Text.Length, 0);
                    return;
                }
            }

        }

        // speech recognition button click 
        private void button8_Click(object sender, EventArgs e)
        {
            sRecog.RecognizeAsyncStop();
            button7.Enabled = true;
            button8.Enabled = false;
            textBox4.Clear();
            System.Media.SystemSounds.Asterisk.Play();
        }

        private void textBox4_TextChanged(object sender, EventArgs e)
        {

        }

        private void label7_Click(object sender, EventArgs e)
        {

        }

        private void groupBox4_Enter(object sender, EventArgs e)
        {

        }

        private void chart1_Click(object sender, EventArgs e)
        {

        }

        private void groupBox6_Enter(object sender, EventArgs e)
        {

        }


        private void textBox7_TextChanged(object sender, EventArgs e)
        {

        }

        private void textBox6_TextChanged(object sender, EventArgs e)
        {

        }

    }
}
